import { useNavigate, useSearchParams } from "@solidjs/router";
import {
  Component,
  Show,
  createSignal,
  createUniqueId,
  onMount,
} from "solid-js";
import { acceptAccountViaAuthCode } from "./stores";
import { $settings } from "../settings/stores";
import { useDocumentTitle } from "../utils";
import cards from "../material/cards.module.css";
import { LinearProgress } from "@suid/material";
import Img from "../material/Img";
import { createRestAPIClient } from "masto";
import { Title } from "../material/typography";

type OAuth2CallbackParams = {
  code?: string;
  error?: string;
  error_description?: string;
};

const MastodonOAuth2Callback: Component = () => {
  const progressId = createUniqueId();
  const titleId = createUniqueId();
  const [params] = useSearchParams<OAuth2CallbackParams>();
  const navigate = useNavigate();
  const setDocumentTitle = useDocumentTitle("Back from Mastodon...");
  const [siteImg, setSiteImg] = createSignal<{
    src: string;
    srcset?: string;
    blurhash: string;
  }>();
  const [siteTitle, setSiteTitle] = createSignal("the Mastodon server");

  onMount(async () => {
    const onGoingOAuth2Process = $settings.get().onGoingOAuth2Process;
    if (!onGoingOAuth2Process) return;
    const client = createRestAPIClient({
      url: onGoingOAuth2Process,
    });
    const ins = await client.v2.instance.fetch();
    setDocumentTitle(`Back from ${ins.title}...`);
    setSiteTitle(ins.title);

    const srcset = []
    if (ins.thumbnail.versions["@1x"]) {
      srcset.push(`${ins.thumbnail.versions["@1x"]} 1x`)
    }
    if (ins.thumbnail.versions["@2x"]) {
      srcset.push(`${ins.thumbnail.versions["@2x"]} 2x`)
    }

    setSiteImg({
      src: ins.thumbnail.url,
      blurhash: ins.thumbnail.blurhash,
      srcset: srcset ? srcset.join(",") : undefined,
    });
  });

  onMount(async () => {
    const onGoingOAuth2Process = $settings.get().onGoingOAuth2Process;
    if (onGoingOAuth2Process && params.code) {
      const acct = await acceptAccountViaAuthCode(
        onGoingOAuth2Process,
        params.code,
      );
      $settings.setKey('onGoingOAuth2Process', undefined)
      navigate('/', {replace: true})
      return;
    }

    const error =
      params.error ||
      (onGoingOAuth2Process ? "unknown" : "oauth2_unknown_target");
    const errorDescription =
      params.error_description ||
      (error === "unknown"
        ? "Remote server sends nothing"
        : error === "oauth2_unknown_target"
          ? "Unknown OAuth2 target. This is an internal error. Please contract the application maintainer."
          : undefined);
    const urlParams = new URLSearchParams({
      error,
    });
    if (errorDescription) {
      urlParams.set("errorDescription", errorDescription);
    }
    navigate("/accounts/sign-in?" + urlParams.toString(), {
      replace: true,
    });
  });
  return (
    <div class={cards.layoutCentered}>
      <div class={cards.card} aria-busy="true" aria-describedby={progressId}>
        <LinearProgress
          class={[cards.cardNoPad, cards.cardGutSkip].join(' ')}
          id={progressId}
          aria-labelledby={titleId}
        />
        <Show when={siteImg()} fallback={<i aria-busy="true" aria-label="Preparing image..." style={{"height": "235px", display: "block"}}></i>}>
          <Img
            src={siteImg()?.src}
            srcset={siteImg()?.srcset}
            blurhash={siteImg()?.blurhash}
            class={[cards.cardNoPad, cards.cardGutSkip].join(' ')}
            alt={`Banner image for ${siteTitle()}`}
            style={{"height": "235px", "display": "block"}}
          />
        </Show>

        <Title component="h6" id={titleId}>
          Contracting {siteTitle}...
        </Title>
        <p>
          If this page stays too long, you can close this page and sign in again.
        </p>
      </div>
    </div>
  );
};

export default MastodonOAuth2Callback;
